==> bu-3.2.2-beta.tar.gz: bu-3.2.2-beta/bu_lib.pm <==
# bu-lib.pm  -  Library functions for bu
# Copyright (C) 2002  Vincent Stemen <bu@hightek.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: bu_lib.pm,v 1.3 2002/06/04 10:11:42 vince Exp $


use Cwd;
use FileHandle;
use File::Find;
use File::Copy;
use File::Basename;
use English;

$CWD = getcwd();                   # Current working directory
$SECS_IN_DAY = 86400;


# bu()
# Usage: bu($file|$directory)
# Writes the file or directory name to the archive stream if the one in the
# backup filesystem is older or does not exist or if bu is not in incremental
# mode.

sub bu($)
{
  my $srcfile = shift;
  my $destfile;
  my $src_modtime     = 0;          # flags
  my $dest_modtime    = 0;
  my $destfile_exists = 0;
  my $srcdir          = 0;
  my $destdir         = 0;

  chomp($srcfile);
  $destfile = "$BACKUP_DIR$srcfile";

  if (-l $srcfile) {
    $src_modtime = ($BASETIME - (lstat(_))[9]) / $SECS_IN_DAY;
    }
  else
    {
    $src_modtime = -M $srcfile;
    -d _ && ($srcdir = 1);
    }

  if (-l $destfile)
    {
    $dest_modtime = ($BASETIME - (lstat(_))[9]) / $SECS_IN_DAY;
    $destfile_exists = 1;
    }
  else
    {
    $dest_modtime = -M $destfile;
    -e _ && ($destfile_exists = 1);
    -d _ && ($destdir = 1);
    }

  if (! $Incremental ||
      ! $destfile_exists ||
      ! ($srcdir && $destdir) && $src_modtime < $dest_modtime)
    {
    # If the source filename has previously been backed us as a
    # directory but is no longer a directory, then remove the old
    # directory from the backup filesystem so that the new source file
    # can be backed up.

    if ($destdir && ! $srcdir)
      {
      if (system("rm -rf $destfile"))
        {
        errmsg("bu(): Error: Could not remove $destfile.\n$!\n");
        return(0);
        }
      }

    $Verbose && print "$srcfile -> $destfile\n";
    mk_parent_directories($srcfile) || return(0);
    print ASTREAM "$srcfile\n";
    }

  return(1);
}



# mk_parent_directories()
# Usage: mk_parent_directories($path)
# Create the leading directories for path in $BACKUP_DIR.
# if $path is /a/b/c, it will create the directories $BACKUP_DIR/a and
# $BACKUP_DIR/a/b.  The owner and permissions is set to match that of
# the original directories.

sub mk_parent_directories($)
{
  my $directory = dirname(shift);
  my $dir;
  my $component;
  my $destpath;
  my ($dev,$ino,$mode,$nlink,$uid,$gid);

  -d "$BACKUP_DIR/$directory" && return(1);

  foreach $component (split('/', $directory))
    {
    $dir .= $dir ? "/$component" : $component;
    $destpath = "$BACKUP_DIR/$dir";
    if (-l $destpath) {
      unlink($destpath);
      }
    if (! -d $destpath)
      {
      -e _ && unlink($destpath);
      ($dev,$ino,$mode,$nlink,$uid,$gid) = stat("/$dir");
      if (! mkdir($destpath, 0777))
        {
        errmsg("mk_parent_directories(): Error: Could not mkdir $destpath.\n$!\n");
        return(0);
        }
      chown($uid, $gid, $destpath);
      chmod($mode & 07777, $destpath);
      }
    }
  return(1);
}


sub errmsg
{
  my $string = shift;

  (! $string) && return;
  print STDERR "$string";
  print ERRLOG "$string";
}


# fgrep()
# Usage: fgrep($expression, $file)
# Returns a list of lines in file that match $expression

sub fgrep($$)
{
  my ($expr, $file) = @_;
  my @lines = ();

  open(FILE, "$file") || die "Cannot open $file\n$!";
  while (<FILE>) {
    /$expr/ && push(@lines, $_);
    }
  close(FILE);
  return(@lines);
}


# generate_filter()
# Usage: @filter = generate_filter($file)
# Read a list of filename patterns from $file, translate them to
# regular expressions and return a list of regular expressions.

sub generate_filter($)
{
  my $file = shift;
  my @filter;

  (! -s $file) && return(0);
  open(FILE, "$file") || die "Cannot read $file\n$!\n";
  foreach (<FILE>)
    {
    /^[ \t]*(#|$)/ && next;  # Ignore comments and blank lines
    chomp;
    s/^[ \t]*//;      # Remove leading and trailing white space
    s/[ \t]*$//;
    s/"//g;           # Strip off quotes if any
    ! -d && s/$/\$/;  # Add a '$' metacharacter to the end if not a directory
    s/\./\\./g;       # Translate shell wildcards to regular expressions
    s/\*/.*/g;
    s/\?/./g;
    s/^/\^/;
    push(@filter, $_);
    }

  close(FILE);
  return(@filter);
}


# get_filelist()
# Usage: $filelist = get_filelist(@dirs)
# This is called once for the entire backup.
# Spawns a child process that scans the specified directories and
# generates a file list to be backed up.  The list is screened based
# on the backup type (ie. full, differential, or incremental) and
# filtered through $EXCLUDE_FILE if $Backup_image is not set.  It
# returns a file descriptor to read the list from the pipe.

sub get_filelist(@)
{
  my    (@dirlist) = @_;
  my    $timestamp = "$TMP/timestamp";
  my    $date;
  my    $dirname;
  local $dir;
  local ($rpipe, $wpipe) = FileHandle::pipe;
  local $expression;


  # _filter() screens the files based on backup mode, last dump date,
  # and One_file_system mode (whether or not to traverse mount
  # points), filters them through @Filter, and streams the filenames
  # into the pipe.

  sub _filter
    {
    my $mod_date;

    if ($Dump)
      {
      if (-l) {
        # Equivalent of -M on a symbolic link
        $mod_date = ($BASETIME - (lstat(_))[9]) / $SECS_IN_DAY;
        }
      else {
        $mod_date = -M $_;
        }

      if (! $Full && $mod_date > $last_dump_date) { return }
      /^\.$/ && return;
      }

    if ($One_file_system && (stat($_))[0] != $File::Find::topdev) {
      $File::Find::prune = 1;
      }
    else {
      $File::Find::prune = 0;
      }

    if ($Backup_image) {
      $File::Find::name =~ s|^\./||;
      }
    else
      {
      # If not dumping a backup file system, use the exclusion filter
      foreach $expression (@Filter)
        {
        if ($File::Find::name =~ m|$expression|)
          {
          if (-d $_) { $File::Find::prune = 1; }
          return;
          }
        }
      }

    print $wpipe "$File::Find::name\n";
    }


  if (! $Dump) { $Backup_image = 0; }
  my $pid = fork();
  defined($pid) || die "Cannot fork\n$!\n";

  if ($pid == 0)                # Child
    {
    $SIG{INT}  = 'DEFAULT';
    $SIG{TERM} = 'DEFAULT';
    close($rpipe);

  DIRECTORY:
    for $dir (@dirlist)
      {
      $dirname = $dir;

      # Filter the directory name through the exclusion list
      foreach $expression (@Filter) {
        $dir =~ m|$expression| && next DIRECTORY;
        }

      if ($Dump && ! $Full)
        {
        if ($last_dump_date = get_dump_date($dir))
          {
          system("touch --date \"$last_dump_date\" $timestamp");
          $last_dump_date = -M $timestamp;
          unlink($timestamp);
          }
        }

      print "Archiving $dirname\n" if ($Dump && ! $Verbose);
      if ($Backup_image) { $dir = '.' }
      $date = scalar localtime();
      find(\&_filter, $dir);
      if ($Dump && ! $Test) { set_dump_date($dirname, $date); }
      }

    exit;
    }

  else                          # Parent
    {
    close($wpipe);
    return($rpipe);
    }
}


sub eject_cd()
{
  if ($SYS_TYPE =~ /BSD/) {
    system("cdcontrol -f $DEVICE eject");
    }
}


# close_cd()
# Usage: close_cd([delay])
# Delay is optional.  It defaults to 10 seconds delay before returning to
# allow time for the drive to become ready after closing.

sub close_cd(;$)
{
  my $sec = shift;
  my $seconds = defined($sec) ? $sec : 10;

  if ($SYS_TYPE =~ /BSD/) {
    system("cdcontrol -f $DEVICE close");
    }

  sleep($seconds);  # Allow time for the drive to become ready
}

# resetcd()
# Usage: resetcd([delay])
# Opens and closes the CD tray.
# Delay is optional.  It defaults to 10 seconds delay before returning to
# allow time for the drive to become ready after closing.

sub reset_cd(;$)
{
  my $seconds = shift;
  print "Reseting CD writer\n";
  eject_cd();
  close_cd($seconds);
}


sub blank_cd()
{
  my $testopt = "";
  my @blankcd = (`whoami` =~ /^root$/) ? qw(nice --5) : "";

  if ($SYS_TYPE =~ /BSD/)
    {
    $Test && ($testopt = "-t");
    push(@blankcd, "burncd", $testopt, "-s", $SPEED, "-f", $DEVICE, "blank");
    if (system("@blankcd"))
      {
      sleep(1);
      system("cdcontrol -f $DEVICE reset");
      return(0);
      }
    }

  return(1);
}


sub mount_cd()
{
  my $fstype;

  if ($SYS_TYPE =~ /BSD/) {
    $fstype = "cd9660";
    }
  elsif ($SYS_TYPE eq "Linux") {
    $fstype = "iso9660";
    }

  if (system("mount -t $fstype $DEVICE $CD_MNT"))
    {
    warn "Cannot mount $DEVICE to $CD_MNT";
    return(0);
    }
  else {
    return(1);
    }
}


sub record_time()
{
  open(T, ">$LOCAL_TIME"); print T time(); close(T);
}


sub wait_for_cd()
{
  $wait_time = 11;   # Seconds to wait for the drive to become ready after
                     # the door closes.

  if (-f $LOCAL_TIME)
    {
    open(T, "$LOCAL_TIME"); $pretime = <T>; close(T);
    $curtime = time();
    $elapsed = $curtime - $pretime;
    ($elapsed < $wait_time) && sleep($wait_time - $elapsed);
    }

  unlink($LOCAL_TIME);
}


sub mkfifo($)
{
  my $fifo = shift;

  if (! -p $fifo && system("mkfifo --mode 600 $fifo")) {
    die "Cannot create fifo $fifo\n$!";
    }
}


# run()
# Run the specified system command and exit with the return code if
# anything other than zero is returned from the command.

sub run(@)
{
  my $rc = system("@_") >> 8;
  if ($rc) {
    exit($rc);
    }
  else {
    return(0);
    }
}


# read_file()
# Usage: @lines = read_file($filename)
# Returns the list of lines in the file.
# If the file does not exist, it waits several fractions of a second for
# the file to appear before giving up and returning 0.

sub read_file($)
{
  my $file = shift;
  my @data;

  for (1,2,3,4)
    {
    if (-f $file)
      {
      open(FILE, "$file") || next;
      @data = <FILE>;
      close(FILE);
      return(@data);
      }
    else {
      system("usleep 75000");
      }
    }

  return(0);
}


sub bye
{
  my $signal = shift;

  # If the dump is killed by a signal during the first volume, then
  # remove the log directory.  Otherwise, note it was aborted in the
  # error log.

  if ($signal && defined(<$LOG/info*>)) {
    print ERRLOG "\nDump aborted.\n\n" if (! fgrep("^Dump aborted", $ERRLOG));
    }
  close(ERRLOG);
  if (! -s $ERRLOG) { unlink($ERRLOG); }
  $Dump && rmdir($LOG);   # Remove the dump log directory if it is empty

  if ($signal) {
    exit($signal);
    }
  elsif ($Dump)
    {
    if (-f $ERRLOG)
      {
      print "\nDump warnings\n-------------\n";
      print (read_file($ERRLOG));
      }
    print "Done.\n\n";
    }

  exit(0);
}


sub signal
{
  my $signal = shift;

  # Disable further signals to make sure I can properly clean up
  $SIG{INT}  = 'IGNORE';
  $SIG{TERM} = 'IGNORE';

  if ($signal eq "INT") { bye($RC_SIGINT) }
  elsif ($signal eq "TERM") { bye($RC_SIGTERM) }
}


sub write_file($@)
{
  my ($file, @data) = @_;
  open(FILE, ">>$file") || die "Cannot write to $file\n$!";
  print FILE @data;
  close(FILE);
}


if ($Dump) { $ERRLOG = "$LOG/errors"; }
else { $ERRLOG = "$LOG.errors"; }
open(ERRLOG, ">$ERRLOG") || die "Cannot write to $ERRLOG\n$!\n";

$SIG{INT}  = \&signal;
$SIG{TERM} = \&signal;

1;

==> bu-3.3-beta.tar.gz: bu-3.3-beta/bu_lib.pm <==
# bu-lib.pm  -  Library functions for bu
# Copyright (C) 2002  Vincent Stemen <bu@hightek.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: bu_lib.pm,v 1.5 2002/07/20 04:58:26 vince Exp $


use Cwd;
use FileHandle;
use File::Find;
use File::Copy;
use File::Basename;
use English;

$CWD = getcwd();                # Current working directory
$SECS_IN_DAY = 86400;
$LOCK_SH = 1;                   # Used by flock()
$LOCK_EX = 2;
$LOCK_UN = 8;


# bu()
# Usage: bu($file|$directory)
# Writes the file or directory name to the archive stream if the one in the
# backup filesystem is older or does not exist or if bu is not in incremental
# mode.

sub bu($)
{
  my $srcfile = shift;
  my $destfile;
  my $src_modtime     = 0;          # flags
  my $dest_modtime    = 0;
  my $destfile_exists = 0;
  my $srcdir          = 0;
  my $destdir         = 0;

  chomp($srcfile);
  $destfile = "$BACKUP_DIR$srcfile";

  if (-l $srcfile) {
    $src_modtime = ($BASETIME - (lstat(_))[9]) / $SECS_IN_DAY;
    }
  else
    {
    $src_modtime = -M $srcfile;
    -d _ && ($srcdir = 1);
    }

  if (-l $destfile)
    {
    $dest_modtime = ($BASETIME - (lstat(_))[9]) / $SECS_IN_DAY;
    $destfile_exists = 1;
    }
  else
    {
    $dest_modtime = -M $destfile;
    -e _ && ($destfile_exists = 1);
    -d _ && ($destdir = 1);
    }

  if (! $Incremental ||
      ! $destfile_exists ||
      ! ($srcdir && $destdir) && $src_modtime < $dest_modtime)
    {
    # If the source filename has previously been backed us as a
    # directory but is no longer a directory, then remove the old
    # directory from the backup filesystem so that the new source file
    # can be backed up.

    if ($destdir && ! $srcdir)
      {
      if (system("rm -rf $destfile"))
        {
        errmsg("bu(): Error: Could not remove $destfile.\n$!\n");
        return(0);
        }
      }

    $Verbose && print "$srcfile -> $destfile\n";
    mk_parent_directories($srcfile) || return(0);
    print ASTREAM "$srcfile\n";
    }

  return(1);
}



# mk_parent_directories()
# Usage: mk_parent_directories($path)
# Create the leading directories for path in $BACKUP_DIR.
# if $path is /a/b/c, it will create the directories $BACKUP_DIR/a and
# $BACKUP_DIR/a/b.  The owner and permissions is set to match that of
# the original directories.

sub mk_parent_directories($)
{
  my $directory = dirname(shift);
  my $dir;
  my $component;
  my $destpath;
  my ($dev,$ino,$mode,$nlink,$uid,$gid);

  -d "$BACKUP_DIR/$directory" && return(1);

  foreach $component (split('/', $directory))
    {
    $dir .= $dir ? "/$component" : $component;
    $destpath = "$BACKUP_DIR/$dir";
    if (-l $destpath) {
      unlink($destpath);
      }
    if (! -d $destpath)
      {
      -e _ && unlink($destpath);
      ($dev,$ino,$mode,$nlink,$uid,$gid) = stat("/$dir");
      if (! mkdir($destpath, 0777))
        {
        errmsg("mk_parent_directories(): Error: Could not mkdir $destpath.\n$!\n");
        return(0);
        }
      chown($uid, $gid, $destpath);
      chmod($mode & 07777, $destpath);
      }
    }
  return(1);
}


sub errmsg
{
  my $string = shift;

  (! $string) && return;
  print STDERR "$string";
  append_to_file($ERRLOG, $string);
}


# fgrep()
# Usage: fgrep($expression, $file)
# Returns a list of lines in file that match $expression

sub fgrep($$)
{
  my ($expr, $file) = @_;
  my @lines = ();

  (! -f $file) && return(0);
  open(FILE, "$file") || die "Cannot open $file\n$!";
  while (<FILE>) {
    /$expr/ && push(@lines, $_);
    }
  close(FILE);
  return(@lines);
}


# generate_filter()
# Usage: @filter = generate_filter($file)
# Read a list of filename patterns from $file, translate them to
# regular expressions and return a list of regular expressions.

sub generate_filter($)
{
  my $file = shift;
  my @filter;

  (! -s $file) && return(0);
  open(FILE, "$file") || die "Cannot read $file\n$!\n";
  foreach (<FILE>)
    {
    /^[ \t]*(#|$)/ && next;  # Ignore comments and blank lines
    chomp;
    s/^[ \t]*//;      # Remove leading and trailing white space
    s/[ \t]*$//;
    s/"//g;           # Strip off quotes if any
    ! -d && s/$/\$/;  # Add a '$' metacharacter to the end if not a directory
    s/\./\\./g;       # Translate shell wildcards to regular expressions
    s/\*/.*/g;
    s/\?/./g;
    s/^/\^/;
    push(@filter, $_);
    }

  close(FILE);
  return(@filter);
}


# get_filelist()
# Usage: $filelist = get_filelist(@dirs)
# This is called once for the entire backup.
# Spawns a child process that scans the specified directories and
# generates a file list to be backed up.  The list is screened based
# on the backup type (ie. full, differential, or incremental) and
# filtered through $EXCLUDE_FILE if $Backup_image is not set.  It
# returns a file descriptor to read the list from the pipe.

sub get_filelist(@)
{
  my    (@dirlist) = @_;
  my    $timestamp = "$TMP/timestamp";
  my    $date;
  my    $dirname;
  local $dir;
  local ($rpipe, $wpipe) = FileHandle::pipe;
  local $expression;


  # _filter() screens the files based on backup mode, last dump date,
  # and One_file_system mode (whether or not to traverse mount
  # points), filters them through @Filter, and streams the filenames
  # into the pipe.

  sub _filter
    {
    my $mod_date;

    if ($Dump)
      {
      if (-l) {
        # Equivalent of -M on a symbolic link
        $mod_date = ($BASETIME - (lstat(_))[9]) / $SECS_IN_DAY;
        }
      else {
        $mod_date = -M $_;
        }

      if (! $Full && $mod_date > $last_dump_date) { return }
      /^\.$/ && return;
      }

    if ($One_file_system && (stat($_))[0] != $File::Find::topdev) {
      $File::Find::prune = 1;
      }
    else {
      $File::Find::prune = 0;
      }

    if ($Backup_image) {
      $File::Find::name =~ s|^\./||;
      }
    else
      {
      # If not dumping a backup file system, use the exclusion filter
      foreach $expression (@Filter)
        {
        if ($File::Find::name =~ m|$expression|)
          {
          if (-d $_) { $File::Find::prune = 1; }
          return;
          }
        }
      }

    print $wpipe "$File::Find::name\n";
    }


  if (! $Dump) { $Backup_image = 0; }
  my $pid = fork();
  defined($pid) || die "Cannot fork\n$!\n";

  if ($pid == 0)                # Child
    {
    $SIG{INT}  = 'DEFAULT';
    $SIG{TERM} = 'DEFAULT';
    close($rpipe);

  DIRECTORY:
    for $dir (@dirlist)
      {
      $dirname = $dir;

      # Filter the directory name through the exclusion list
      foreach $expression (@Filter) {
        $dir =~ m|$expression| && next DIRECTORY;
        }

      if ($Dump && ! $Full)
        {
        if ($last_dump_date = get_dump_date($dir))
          {
          system("touch --date \"$last_dump_date\" $timestamp");
          $last_dump_date = -M $timestamp;
          unlink($timestamp);
          }
        }

      print "Archiving $dirname\n" if ($Dump && ! $Verbose);
      if ($Backup_image) { $dir = '.' }
      $date = scalar localtime();
      find(\&_filter, $dir);
      if ($Dump && ! $Test) { set_dump_date($dirname, $date); }
      }

    exit;
    }

  else                          # Parent
    {
    close($wpipe);
    return($rpipe);
    }
}


sub eject_cd()
{
  if ($SYS_TYPE =~ /BSD/) {
    system("cdcontrol -f $DEVICE eject");
    }
}


# close_cd()
# Usage: close_cd([delay])
# Delay is optional.  It defaults to 10 seconds delay before returning to
# allow time for the drive to become ready after closing.

sub close_cd(;$)
{
  my $sec = shift;
  my $seconds = defined($sec) ? $sec : 10;

  if ($SYS_TYPE =~ /BSD/) {
    system("cdcontrol -f $DEVICE close");
    }

  sleep($seconds);  # Allow time for the drive to become ready
}

# resetcd()
# Usage: resetcd([delay])
# Opens and closes the CD tray.
# Delay is optional.  It defaults to 10 seconds delay before returning to
# allow time for the drive to become ready after closing.

sub reset_cd(;$)
{
  my $seconds = shift;
  print "Reseting CD writer\n";
  eject_cd();
  close_cd($seconds);
}


sub blank_cd()
{
  my $testopt = "";
  my @blankcd = (`whoami` =~ /^root$/) ? qw(nice --5) : "";

  if ($SYS_TYPE =~ /BSD/)
    {
    $Test && ($testopt = "-t");
    push(@blankcd, "burncd", $testopt, "-s", $SPEED, "-f", $DEVICE, "blank");
    if (system("@blankcd"))
      {
      sleep(1);
      system("cdcontrol -f $DEVICE reset");
      return(0);
      }
    }

  return(1);
}


sub mount_cd()
{
  my $fstype;

  if ($SYS_TYPE =~ /BSD/) {
    $fstype = "cd9660";
    }
  elsif ($SYS_TYPE eq "Linux") {
    $fstype = "iso9660";
    }

  if (system("mount -t $fstype $DEVICE $CD_MNT"))
    {
    warn "Cannot mount $DEVICE to $CD_MNT";
    return(0);
    }
  else {
    return(1);
    }
}


sub record_time()
{
  open(T, ">$LOCAL_TIME"); print T time(); close(T);
}


sub wait_for_cd()
{
  $wait_time = 11;   # Seconds to wait for the drive to become ready after
                     # the door closes.

  if (-f $LOCAL_TIME)
    {
    open(T, "$LOCAL_TIME"); $pretime = <T>; close(T);
    $curtime = time();
    $elapsed = $curtime - $pretime;
    ($elapsed < $wait_time) && sleep($wait_time - $elapsed);
    }

  unlink($LOCAL_TIME);
}


sub mkfifo($)
{
  my $fifo = shift;

  if (! -p $fifo && system("mkfifo -m 600 $fifo")) {
    die "Cannot create fifo $fifo\n$!";
    }
}


# run()
# Run the specified system command and exit with the return code if
# anything other than zero is returned from the command.

sub run(@)
{
  my $rc = system("@_") >> 8;
  if ($rc) {
    exit($rc);
    }
  else {
    return(0);
    }
}


# read_file()
# Usage: @lines = read_file($filename)
# Returns the list of lines in the file.
# If the file does not exist or has no data in it, it returns 0.
# It locks the file with flock() before reading.

sub read_file($)
{
  my $file = shift;
  my @data;

  if (-f $file)
    {
    if (! open(FILE, "$file"))
      {
      errmsg "Cannot open $file for reading\n$!\n";
      return(0);
      }
    else
      {
      flock(FILE, $LOCK_SH);
      @data = <FILE>;
      flock(FILE, $LOCK_UN);
      close(FILE);
      return((@data) ? @data : 0);
      }
    }
  else
    {
    $DEBUG && print DLOG "\nread_file(): $file does not exist\n";
    return(0);
    }
}


sub bye
{
  my $signal = shift;

  # If the dump is killed by a signal during the first volume, then
  # remove the log directory.  Otherwise, note it was aborted in the
  # error log.

  if ($signal && defined(<$LOG/info*>)) 
    {
    if (! fgrep("^Dump aborted", $ERRLOG)) {
      append_to_file($ERRLOG, "\nDump aborted.\n\n");
      }
    }
 
  if (! -s $ERRLOG) { unlink($ERRLOG); }
  $Dump && rmdir($LOG);   # Remove the dump log directory if it is empty

  if ($signal) {
    exit($signal);
    }
  elsif ($Dump)
    {
    if (-f $ERRLOG)
      {
      print "\nDump warnings\n-------------\n";
      print (read_file($ERRLOG));
      }
    print "Done.\n\n";
    }

  exit(0);
}


sub signal
{
  my $signal = shift;

  # Disable further signals to make sure I can properly clean up
  $SIG{INT}  = 'IGNORE';
  $SIG{TERM} = 'IGNORE';

  if ($signal eq "INT") { bye($RC_SIGINT) }
  elsif ($signal eq "TERM") { bye($RC_SIGTERM) }
}


# write_to_file()
# Usage: write_to_file($file, @data)
# Locks $file with flock() then writes @data to it.

sub write_to_file($@)
{
  my ($file, @data) = @_;
  open(FILE, ">$file") || die "Cannot write to $file\n$!";
  flock(FILE, $LOCK_EX);
  print FILE @data;
  close(FILE);
}


# append_to_file()
# Usage: append_to_file($file, @data)
# Locks $file with flock() then appends @data to it.

sub append_to_file($@)
{
  my ($file, @data) = @_;
  open(FILE, ">>$file") || die "Cannot write to $file\n$!";
  flock(FILE, $LOCK_EX);
  print FILE @data;
  close(FILE);
}


if ($Dump) { $ERRLOG = "$LOG/errors"; }
else { $ERRLOG = "$LOG.errors"; }

$SIG{INT}  = \&signal;
$SIG{TERM} = \&signal;

1;
